-- ===============================================================================
-- StairsPlus Bonus!
-- ===============================================================================
--[[
This section of code that makes blocks compatible with MoreBlocks' circular saw.
I've added circular saw compatible code for default snowblocks and ice. :D
A big thanks to Calinou and ShadowNinja for making this possible.

Because StairsPlus creates partial blocks, it didn't seem quite right that the
smallest microblocks would produce a full-sized water_source node when melted.
So I toned them down a bit by changing their melt to a temporary,
2-second water_source. See "melts" in abms.lua file for the various intensities.

___...::: ATTENTION MINETEST SERVER OPERATORS :::...___
You may or may not have noticed in your server logs that MoreBlocks stairs/slabs/
panels/microblocks are not recorded as to when, who, what, and where. This is
important information when trying to determine if a player who dug these blocks
is the owner (the player who placed the block) or is a griefer stealing the block.

There is an option that will log when these blocks are placed but it comes at the
cost of losing the auto-rotation of those blocks when placed. They can still be
rotated with a screwdriver but if screwdrivers are disabled on your server your
players won't be able to position MoreBlocks, saw-made blocks.

To enable logging the placement of these blocks, un-comment these lines:

--on_place = minetest.item_place

There is one in each of the "stairsplus.register_all" sections.

~ LazyJ
-- ===============================================================================
--]]

--snow_stairsplus = {}

if (minetest.get_modpath("moreblocks"))

then

-- Default snowblock and ice stairs/slabs/panels/microblocks.

	local ndef = minetest.registered_nodes["default:ice"]
	local groups = {}
	for k, v in pairs(ndef.groups) do groups[k] = v end

	stairsplus:register_all("moreblocks", "ice", "default:ice", {
		description = ndef.description,
		paramtype2 = "facedir",
		-- Added "icemaker=1" in groups. This ties into the freezing
		-- function in the ABMs.lua file. ~ LazyJ
		groups = {cracky=1, crumbly=1, choppy=1, oddly_breakable_by_hand=1, melts=2, icemaker=1},
		sounds = default.node_sound_glass_defaults(),
		tiles = ndef.tiles,
		-- Because of the "use_texture_alpha" line, that gives ice transparency, I couldn't combine
		-- default ice and default snowblocks in a list like MoreBlocks does. ~ LazyJ
		use_texture_alpha = true,
		sunlight_propagates = true,
		-- This "on_place" line makes placing these nodes recorded in the logs.
		-- Useful for investigating griefings and determining ownership
		-- BUT these nodes will nolonger auto-rotate into position. ~ LazyJ

		--on_place = minetest.item_place,

 		-- The "on_construct" part below, thinking in terms of layers, dirt_with_snow could
 		-- also double as dirt_with_frost which adds subtlety to the winterscape. ~ LazyJ
		on_construct = function(pos)
			pos.y = pos.y - 1
			if minetest.get_node(pos).name == "default:dirt_with_grass"
			or minetest.get_node(pos).name == "default:dirt" then
				minetest.set_node(pos, {name="default:dirt_with_snow"})
			end
		end
	})
--end


	local ndef = minetest.registered_nodes["default:snowblock"]
	local groups = {}
	for k, v in pairs(ndef.groups) do groups[k] = v end

	stairsplus:register_all("moreblocks", "snowblock", "default:snowblock", {
		description = ndef.description,
		paramtype2 = "facedir",
		-- Added "icemaker=1" in groups. This ties into the freezing function
		-- in the ABMs.lua file. ~ LazyJ
		groups = {cracky=3, crumbly=3, choppy=3, oddly_breakable_by_hand=3, melts=2, icemaker=1},
		tiles = ndef.tiles,
		sunlight_propagates = true,
		sounds = default.node_sound_dirt_defaults({
			footstep = {name="default_snow_footstep", gain=0.25},
			dig = {name="default_dig_crumbly", gain=0.4},
			dug = {name="default_snow_footstep", gain=0.75},
			place = {name="default_place_node", gain=1.0}
		}),
		--on_place = minetest.item_place,
		on_construct = function(pos)
			pos.y = pos.y - 1
			if minetest.get_node(pos).name == "default:dirt_with_grass"
			or minetest.get_node(pos).name == "default:dirt" then
				minetest.set_node(pos, {name="default:dirt_with_snow"})
			end
		end
	})

-- Snow stairs/slabs/panels/microblocks.

local snow_nodes = {
	"snow_brick",
	"snow_cobble",
}

for _, name in pairs(snow_nodes) do
	local nodename = "snow:"..name
	local ndef = minetest.registered_nodes[nodename]
	local groups = {}
	for k, v in pairs(ndef.groups) do groups[k] = v end

	stairsplus:register_all("moreblocks", name, nodename, {
		description = ndef.description,
		groups = {cracky=2, crumbly=2, choppy=2, oddly_breakable_by_hand=2, melts=2, icemaker=1},
		tiles = ndef.tiles,
		--paramtype2 = "facedir",
		sunlight_propagates = true,
		sounds = default.node_sound_dirt_defaults({
			footstep = {name="default_snow_footstep", gain=0.25},
			dig = {name="default_dig_crumbly", gain=0.4},
			dug = {name="default_snow_footstep", gain=0.75},
			place = {name="default_place_node", gain=1.0}
		}),
		--on_place = minetest.item_place,
		on_construct = function(pos)
			pos.y = pos.y - 1
			if minetest.get_node(pos).name == "default:dirt_with_grass"
				or minetest.get_node(pos).name == "default:dirt"
				then minetest.set_node(pos, {name="default:dirt_with_snow"})
			end
		end,
	})
end
else
return
end
